@@ -1,10 +1,10 @@
 test_that("dep_graph_next_package finds next installable package", {
   # nolint start, styler: off
   g <- igraph::make_graph(~
-    A +- B +- C,
-    A +------ D,
-    A +- E +- D,
-    A +- F +- D
+                            A -+ B -+ C,
+                          A ------+ D,
+                          A -+ E -+ D,
+                          A -+ F -+ D
   )
   # nolint end, styler: on
 
@@ -21,12 +21,14 @@ test_that("dep_graph_next_package finds next installable package", {
   V(g)["D"]$status <- STATUS[["in progress"]]
   V(g)["C"]$status <- STATUS[["done"]]
   g <- task_graph_update_ready(g)
-  expect_equal(names(task_graph_which_ready(g)), "B")
+  ready_nodes <- V(g)[V(g)$status == STATUS$ready]
+  expect_equal(names(ready_nodes), "B")
 
   # if the order is reversed, now "F" and "E" should be next
   V(g)$status <- STATUS[["pending"]]
   V(g)["D"]$status <- STATUS[["done"]]
   V(g)["C"]$status <- STATUS[["in progress"]]
   g <- task_graph_update_ready(g)
-  expect_equal(names(task_graph_which_ready(g)), c("F", "E"))
+  ready_nodes <- V(g)[V(g)$status == STATUS$ready]
+  expect_equal(names(ready_nodes), c("F", "E"))
 })
